#!/usr/bin/perl
#
# 	Component: PD - i500 Interworking
#
#	Primary Author: Winton Campbell
#
#	Peer Review:
#
#	Copyright  (C) Tivoli Systems/IBM. 2000
#	Unpublished Work
#	All Rights Reserved
#	Licensed Material - Property of Tivoli Systems/IBM
#
#	@(#)75    1.1     src/ivrgy/i500/scripts/i500sup.pl, pd.src, pd, koal-pd   8/16/00   21:04:20
#
#
# =========================================================================
#	FILE:			i500sup.l	
#	AUTHOR:			Winton Campbell
#
#	DESCRIPTION:	A series of utility subroutines used to manipulate
#					i500 objects as follows
#
# ===========================================================================

require 5.001;

# subroutines

sub convert_hostname
{
#	use nslookup to ip address. 
#	Look for string "Name:*<hostname>\nAddress:<hostip>
#	Returns ip address or empty list
	my $hn = $_[0];
	return ($hn);
}
	

sub lookup
{
	# takes LDAP attribute name and returns corresponding i500 macro name if 
	# found otherwise returns attribute
	my $key = @_[0];
	$path = $ENV{"ODSRELEASE"};
	# lookup the oids and local oids files for the macro name for this attribute
	$filter = "=A.*\\s$key(\\s|\\\$)";
	open OIDFILE, "$path/scripts/oids";
	@line = <OIDFILE>;
	close OIDFILE;
	( $line ) = grep(/$filter/, @line);
	if ( ! $line ) {
		open OIDFILE, "$path/scripts/oidslocal";
		@line = <OIDFILE>;
		close OIDFILE;
		( $line ) = grep(/$filter/, @line);
		if ( ! $line ) { return $key; }
	}
	# extract macro name - starts with =A 
	@mac = split(' ', $line);
	@mac[3]=~s/=A//;
	return @mac[3];
}

sub add_quit
{
	# add script quit macro to file handle's file
	$fh = shift;
	print $fh ".\n";
}

sub create_macro_dn
{
	# takes a list of LDAP DNs and returns a list of N() macro equivalents
	# for each DN split into RDNs, reverse their order, convert to macro form
	# and wrap result in N() macro
	my @plist  = @_;
	foreach my $raw_dn (@plist) 
	{
		# skip blank line separator
		next if ( $raw_dn eq "\n" ); 
		$raw_dn =~s/^.*: //;
		@rdn = split_ldap_dn($raw_dn);
		$dn = "N(";
		# reverse thru list to get correct N() macro order
		$first = 'Y';
		foreach my $rdn (reverse(@rdn))
		{
			if ( $first eq 'N' ) { $dn .= ","; }
			$first = "N";
			$dn .= $rdn;
		}
		$dn .= ")";
		$raw_dn = $dn;
	}
	return @plist;
}

sub split_ldap_dn
{
	# Splits the LDAP DN up into RDNs.

	$_ = $_[0];
	@reversedchars = split('',$_);
	@chars = reverse(@reversedchars);
	$quoted = 0;
	$escaped = 0;
	$reset_escaped = 1;
	$rdn = "SET[SEQ[A";
	$divert = 1;
	@rdns = ();
	while (defined($char = pop(@chars)))
	{
		if ($char eq "\\" && $escaped)
		{
			$escaped = 0;
			$rdn = "$rdn$char";
		}
		elsif ($char eq "\\")
		{
			$escaped = 1;
			$reset_escaped = 0;
			$rdn = "$rdn$char";
		}
		elsif ($char eq "\"" && ! $quoted && ! $escaped)
		{
			$quoted = 1;
		}
		elsif ($char eq "\"" && $quoted && ! $escaped)
		{
			$quoted = 0;
		}
		elsif (($char eq "," || $char eq ";") && ! $quoted && ! $escaped)
		{
			$rdn .= "\"]]]";
			push(@rdns,$rdn);
			$rdn = "SET[SEQ[A";
			$divert = 1;
		}
		elsif (($char eq "," || $char eq ";" || $char eq "=" || $char eq "<" ||
			$char eq ">" || $char eq "#" || $char eq "+" ) && $escaped)
		{
			chop($rdn);
			$rdn = "$rdn$char";
		}
		else
		{
			if ( $char eq "=" )
			{
				( $mname ) = lookup($diversion);
				$rdn .= $mname;
				if ( $mname eq "ctryName") { $tag="PRS"; }
				else { $tag="T61S"; }
				$rdn .= " $tag\[=\"";
				$diversion = "";
				$divert = 0;
			}
			elsif ( $char ne "\n" )
			{
				if ( $divert ) { 
					$diversion .= $char; 
				} else {
					$rdn .= $char;
				}
			}
		}
		if ($escaped && $reset_escaped)
		{
			$escaped = 0;
		}
		$reset_escaped = 1;
	}
	if ( $rdn && $rdn ne "SET[SEQ[A" )
	{
		$rdn .= "\"]]]";
		push(@rdns,$rdn);
	}
	return @rdns;
}

sub add_file_hdr
{
	# takes a filehandle as input
	# creates a standard header for a macro file including bind info
	$fh = shift;
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	$year += 1900;										# year is from 1900
	$mon++;
	#
	# now get optional DAP port environment value
	#
	my $newdap = $ENV{"I500_DAP_PORT"};
	if ($newdap=~/^\d+$/) { $dap=$newdap; } # only integer is valid
	( $dapstr ) = check_i500_host();
	print $fh "{\n\tDSA generation script\n\n";
	print $fh "\tcreated at $hour:$min:$sec on $mday/$mon/$year\n";
	print $fh "\tby user $opt_D\n}\n\n";
	print $fh "bind -n $admin_name -p $admin_pw & \n ";
	print $fh "	-tpath Vaddr_list(Vrem_comms($hostip,$dapstr)) \n";
}

sub check_i500_host
{
	# attempt bind to i500 server assuming Unix host
	# if it fails assume NT host
	my $tmp_file = "bind.tmp";
	open TMP_BIND_FILE, ">$tmp_file";
	print TMP_BIND_FILE "bind -n $admin_name -p $admin_pw & \n ";
	# <dap> is in Unix port position
	print TMP_BIND_FILE "	-tpath Vaddr_list(Vrem_comms($hostip,$dap)) \n";
	close TMP_BIND_FILE;
	$outp=qx($list_cmd $tmp_file | odsdua$exext);
	unlink ($tmp_file);
	if ($outp=~/_ABORT/) { return ",$dap"; }
	return $dap; 

}
sub check_nt 
{
	# tests BUILDTYPE for WIN32 and sets global NT flag if true
	$NT=0;
	my $bt=$ENV{"OS"};
	if ($bt eq "Windows_NT") { $NT=1; }
	return ( $NT );
}

sub set_files
{
	( $nt ) = check_nt();
	if ($nt == 1)
	{
		return ( "type", ".exe" , "nul:" );
	}
	else
	{
		return ( "cat", "" , "/dev/null" );
	}
}

sub run_dua
{
	#
	# pipe a file into odsdua then remove temporary files
	#
	my( $lcmd, $file, $xt ) = @_;
	$outp=qx($lcmd $file | odsdua$ext);
	unlink ( $file, "du.000" );
	rmdir ( "temp" );
}
