/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _AZN_SVC_PROTOS_H
#define _AZN_SVC_PROTOS_H

#ifdef _WIN32
#undef AZN_DECLSPEC
#define AZN_DECLSPEC    __declspec(dllexport)
#else
#undef AZN_DECLSPEC
#define AZN_DECLSPEC
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*
 * The AZN service initialize interface.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_initialize(
  const int                   argc,       /* in */
  const char                  **argv,     /* in */
  const azn_attrlist_h_t      svcInit,    /* in */
  azn_attrlist_h_t            *svcInfo    /* out */
);


/*
 * The AZN service shutdown interface.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_shutdown(
  const int                   argc,       /* in */
  const char                  **argv,     /* in */
  const azn_attrlist_h_t      svcInit,    /* in */
  azn_attrlist_h_t            *svcInfo    /* out */
);


/*
 * AZN entitlement service interface prototype.
 * Returns entitlements in the form of AZN attributes
 * to the caller from an external entitlements source.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_entitlement_get_entitlements(
  const azn_creds_h_t      creds                /* in */,
  const azn_string_t       entitlements_svc_id  /* in */,
  const azn_attrlist_h_t   app_context          /* in */,
  azn_attrlist_h_t         *entitlements        /* out */
);


/*
 * AZN credential modification service interface prototype.
 * Manipulates the attributes in a credential based upon
 * an original credential and attributes passed into the 
 * service interface.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_creds_modify(
  const azn_creds_h_t      creds       /* in */,
  const azn_string_t       mod_svc_id  /* in */,
  const azn_attrlist_h_t   mod_info    /* in */,
  azn_creds_h_t            *new_creds  /* out */
);


/*
 * AZN PAC service interface prototype to build a proprietary 
 * formatted PAC, or "Privilege Attribute Certificate" from 
 * an AM authorization credential. 
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_creds_get_pac(
  const azn_creds_h_t   creds       /* in */,
  const azn_string_t    pac_svc_id  /* in */,
  azn_buffer_t          *pac        /* out */
);


/*
 * AZN PAC service interface prototype to build an AM 
 * authorization credential from a proprietary formatted PAC, 
 * or "Privilege Attribute Certificate".
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_pac_get_creds(
  const azn_buffer_t    pac         /* in */,
  const azn_string_t    pac_svc_id  /* in */,
  azn_creds_h_t         *new_creds  /* out */
);


/*
 * External Authorization Service (EAS) interface prototype.
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_decision_access_allowed_ext(
  const azn_creds_h_t      creds               /* in */,
  const azn_string_t       protected_resource  /* in */,
  const azn_string_t       operation           /* in */,
  const azn_attrlist_h_t   app_context         /* in */,
  int                      *permission         /* out */,
  azn_attrlist_h_t         *permission_info    /* out */
);


#ifdef __cplusplus
}
#endif

#endif /* _AZN_SVC_PROTOS_H */

